(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* $Id: genr_pattern.ml,v 1.15 2008/02/11 23:05:59 weis Exp $ *)

open Parsetree
open Longident
;;

let remove_topconstr r =
  match r.ppat_desc with
  | Ppat_construct (_, Some p, _) -> p
  | _ -> assert false
;;

let rec elim_parens p =
  match p.ppat_desc with
  | Ppat_parens p -> elim_parens p
  | _ -> p
;;

let rec add_underscores p =
  { p with ppat_desc = add_underscores_aux p.ppat_desc }

and add_opt = function
  | Some p -> Some (add_underscores p)
  | None -> None

and add_underscores_aux = function
  | Ppat_parens p -> Ppat_parens (add_underscores p)
  | Ppat_any -> Ppat_any
  | Ppat_var _ as pd -> pd
  | Ppat_alias (p, s) -> Ppat_alias (add_underscores p, s)
  | Ppat_constant _ as pd -> pd
  | Ppat_tuple pats -> Ppat_tuple (List.map add_underscores pats)
  | Ppat_construct (id, Some p, b) ->
      begin try
       let _ = Check.find_gident id in
       let args =
         match (elim_parens p).ppat_desc with
         | Ppat_tuple pats -> pats
         | _ -> [p] in
       let args =
         { ppat_desc = Ppat_any; ppat_loc = Location.none }
	 :: List.map add_underscores args in
       let p = { p with ppat_desc = Ppat_tuple args } in
       Ppat_construct (id, Some p, b) with
     | Not_found ->
       Ppat_construct (id, Some (add_underscores p), b)
      end
  | Ppat_construct (l, popt, b) -> Ppat_construct (l, add_opt popt, b)
  | Ppat_variant (label, popt) -> Ppat_variant (label, add_opt popt)
  | Ppat_record idpats ->
    let f (id,pat) = (id, add_underscores pat) in
    Ppat_record (List.map f idpats)
  | Ppat_array pats ->
    Ppat_array (List.map add_underscores pats)
  | Ppat_or (p1, p2) ->
    Ppat_or (add_underscores p1, add_underscores p2)
  | Ppat_constraint (p, ct) ->
    Ppat_constraint (add_underscores p, ct)
  | Ppat_type _ as pd -> pd
;;
