(**********************************************************************)
(*                                                                    *)
(*                           Moca                                     *)
(*                                                                    *)
(*          Pierre Weis, INRIA Rocquencourt                           *)
(*          Frdric Blanqui, projet Protheo, INRIA Lorraine          *)
(*                                                                    *)
(*  Copyright 2005-2008,                                              *)
(*  Institut National de Recherche en Informatique et en Automatique. *)
(*  All rights reserved.                                              *)
(*                                                                    *)
(*  This file is distributed under the terms of the Q Public License. *)
(*                                                                    *)
(**********************************************************************)

(* substitutions *)

open Var;;
open Term;;
open Symb;;

type subs;;

(* identity: substitution with no binding *)
val empty : subs;;

(* test if a substitution is the identity *)
val is_empty : subs -> bool;;

(* one-variable substitution *)
val subs : var -> term -> subs;;

(* add a new binding: replace the previous one if any *)
val add : var -> term -> subs -> subs;;

(* add to s the mapping (x,t) if mem s x = false or find s x = t.
raise Failure otherwise *)
val check_add : var -> term -> subs -> subs;;

(* raise Not_found if var is not bound *)
val find : var -> subs -> term;;

(* test if there is a binding *)
val mem : var -> subs -> bool;;

(* map a function on bindings *)
val map : (term -> term) -> subs -> subs;;

(* apply a substitution to a term *)
val apply : subs -> term -> term;;

(* domain of a substitution *)
val domain : subs -> VarSet.t;;

(* restriction of a susbtitution to some domain *)
val restrict : subs -> VarSet.t -> subs;;

(* composition of 2 substitutions:
apply (comp s1 s2) t = apply s1 (apply s2 t) *)
val comp : subs -> subs -> subs;;

(* restricted composition: comp_res s1 s2 = restrict (domain s2) (comp s1 s2) *)
val comp_res : subs -> subs -> subs;;

(* inverse a substitution mapping variables to variables.
raise Failure otherwise *)
val inverse : subs -> subs;;

(* print a substitution into a buffer *)
val bprint : Buffer.t -> subs -> unit;;
